import agate
from matplotlib import pyplot

from agatecharts.charts import Bars, Columns, Lines, Scatter
from agatecharts.utils import round_limits

#: Default rendered chart size in inches
DEFAULT_SIZE = (8, 8)

#: Default rendered chart dpi
DEFAULT_DPI = 72


def bar_chart(self, label_column_name, value_column_names, filename=None, size=DEFAULT_SIZE, dpi=DEFAULT_DPI):
    """
    Plots a bar chart.

    See :meth:`agatecharts.table.plot` for an explanation of keyword arguments.

    :param label_column_name: The name of a column in the source to be used for
        the vertical axis labels. Must refer to a column containing
        :class:`.Text`, :class:`.Number` or :class:`.Date` data.
    :param value_column_names: One or more column names in the source, each of
        which will used to define the horizontal width of a bar. Must refer to a
        column containing :class:`.Number` data.
    """
    chart = Bars(label_column_name, value_column_names)

    plot(self, chart, filename, size, dpi)


def column_chart(self, label_column_name, value_column_names, filename=None, size=DEFAULT_SIZE, dpi=DEFAULT_DPI):
    """
    Plots a column chart.

    See :meth:`agatecharts.table.plot` for an explanation of keyword arguments.

    :param label_column_name: The name of a column in the source to be used for
        the horizontal axis labels. Must refer to a column containing
        :class:`.Text`, :class:`.Number` or :class:`.Date` data.
    :param value_column_names: One or more column names in the source, each of
        which will used to define the vertical height of a bar. Must refer to a
        column containing :class:`.Number` data.
    """
    chart = Columns(label_column_name, value_column_names)

    plot(self, chart, filename, size, dpi)


def line_chart(self, x_column_name, y_column_names, filename=None, size=DEFAULT_SIZE, dpi=DEFAULT_DPI):
    """
    Plots a line chart.

    See :meth:`agatecharts.table.plot` for an explanation of keyword arguments.

    :param x_column_name: The name of a column in the source to be used for
        the horizontal axis. May refer to a column containing
        :class:`.Number`, :class:`.Date` or :class:`.DateTime`
        data.
    :param y_column_names: A sequence of column names in the source, each of
        which will be used for the vertical axis. Must refer to a column with
        :class:`.Number` data.
    """
    chart = Lines(x_column_name, y_column_names)

    plot(self, chart, filename, size, dpi)


def scatter_chart(self, x_column_name, y_column_name, filename=None, size=DEFAULT_SIZE, dpi=DEFAULT_DPI):
    """
    Plots a scatter plot.

    See :meth:`agatecharts.table.plot` for an explanation of keyword arguments.

    :param x_column_name: Column containing X values for the points to plot.
        Must refer to a column containg :class:`.Number` data.
    :param y_column_name: Column containing Y values for the points to plot.
        Must refer to a column containg :class:`.Number` data.
    """
    chart = Scatter(x_column_name, y_column_name)

    plot(self, chart, filename, size, dpi)


def plot(table, chart, filename=None, size=DEFAULT_SIZE, dpi=DEFAULT_DPI):
    """
    Execute a plot of this :class:`.Table`.

    This method should not be called directly by the user.

    :param chart: An chart class to render.
    :param filename: A filename to render to. If not specified will render
        to screen in "interactive mode".
    :param size: A (width, height) tuple in inches defining the size of the
        canvas to render to.
    :param dpi: A number defining the pixels-per-inch to render.
    """
    if chart.show_legend():
        size = (
            size[0] * 1.2,
            size[1]
        )

    x_min, x_max = chart.get_x_domain(table)
    y_min, y_max = chart.get_y_domain(table)
    x_min, x_max, y_min, y_max = round_limits(x_min, x_max, y_min, y_max)

    pyplot.figure(figsize=size, dpi=dpi)
    axes = pyplot.subplot(1, 1, 1)

    chart.plot(table, axes)

    pyplot.grid(visible=True, which='major', color='0.85', linestyle='-')
    axes.set_axisbelow(True)

    # matplotlib won't accept Decimal for limit values
    if x_min is not None and x_max is not None:
        axes.set_xlim(float(x_min), float(x_max))

    if y_min is not None and y_max is not None:
        axes.set_ylim(float(y_min), float(y_max))

    if chart.show_legend():
        bbox = axes.get_position()
        axes.set_position([bbox.x0, bbox.y0, bbox.width / 1.2, bbox.height])

        axes.legend(loc='center left', bbox_to_anchor=(1, 0.5))

    if filename:
        pyplot.savefig(filename)
    else:
        pyplot.show()


agate.Table.bar_chart = bar_chart
agate.Table.column_chart = column_chart
agate.Table.line_chart = line_chart
agate.Table.scatter_chart = scatter_chart



import agate

from agatecharts.charts.base import Chart
from agatecharts.colors import Qualitative


class Lines(Chart):
    def __init__(self, x_column_name, y_column_names):
        self._x_column_name = x_column_name

        if isinstance(y_column_names, str):
            y_column_names = [y_column_names]

        self._y_column_names = y_column_names

    def show_legend(self):
        return len(self._y_column_names) > 1

    def get_x_domain(self, table):
        if not isinstance(table.columns[self._x_column_name].data_type, agate.Number):
            return (None, None)

        x_min = min(table.columns[self._x_column_name])
        x_max = max(table.columns[self._x_column_name])

        return (x_min, x_max)

    def get_y_domain(self, table):
        y_min = min([min(table.columns[name]) for name in self._y_column_names])
        y_max = max([max(table.columns[name]) for name in self._y_column_names])

        return (y_min, y_max)

    def plot(self, table, axes):
        colors = Qualitative()
        legend_lines = []

        x_column = table.columns[self._x_column_name]

        if not isinstance(x_column.data_type, agate.Number) and \
                not isinstance(x_column.data_type, agate.Date) and \
                not isinstance(x_column.data_type, agate.DateTime):
            raise ValueError('Only Number, Date and DateTime data are supported for line chart X-axis.')

        for y_column_name in self._y_column_names:
            y_column = table.columns[y_column_name]

            if not isinstance(y_column.data_type, agate.Number):
                raise ValueError('Only Number data is supported for line chart Y-axis.')

            plot_lines = axes.plot(
                x_column,
                y_column,
                linewidth=2,
                color=next(colors),
                label=y_column_name
            )

            legend_lines.append(plot_lines[0])

        axes.set_xlabel(self._x_column_name)

        if len(self._y_column_names) == 1:
            axes.set_ylabel(self._y_column_names[0])

        return (legend_lines, self._y_column_names)



class Chart:
    """
    Base class for a chart type.
    """

    def show_legend(self):
        """
        Returns true if this chart should render a legend. (Usually if there
        is more than one series of data.)
        """
        raise NotImplementedError

    def get_x_domain(self, table):
        """
        Compute the x domain for this chart type for a given table.
        """
        raise NotImplementedError

    def get_y_domain(self, table):
        """
        Compute the y domain for this chart type for a given table.
        """
        raise NotImplementedError

    def plot(self, table, axes):
        """
        Subclasses implement this method to draw a single chart, regardless of
        whether or not it is part of small multiples.
        """
        raise NotImplementedError



import math

import agate
from matplotlib import pyplot

from agatecharts.charts import Bars, Columns, Lines, Scatter
from agatecharts.table import DEFAULT_DPI
from agatecharts.utils import round_limits

#: Default small multiple chart size in inches
DEFAULT_MULTIPLE_SIZE = (4, 4)


def bar_chart(self, label_column_name, value_column_names, filename=None, size=None, dpi=DEFAULT_DPI):
    """
    See :meth:`agatecharts.table.bar_chart`.
    """
    chart = Bars(label_column_name, value_column_names)

    plot(self, chart, filename, size, dpi)


def column_chart(self, label_column_name, value_column_names, filename=None, size=None, dpi=DEFAULT_DPI):
    """
    See :meth:`agatecharts.table.column_chart`.
    """
    chart = Columns(label_column_name, value_column_names)

    plot(self, chart, filename, size, dpi)


def line_chart(self, x_column_name, y_column_names, filename=None, size=None, dpi=DEFAULT_DPI):
    """
    See :meth:`agatecharts.table.line_chart`.
    """
    chart = Lines(x_column_name, y_column_names)

    plot(self, chart, filename, size, dpi)


def scatter_chart(self, x_column_name, y_column_name, filename=None, size=None, dpi=DEFAULT_DPI):
    """
    See :meth:`agatecharts.table.scatter_chart`.
    """
    chart = Scatter(x_column_name, y_column_name)

    plot(self, chart, filename, size, dpi)


def plot(tableset, chart, filename=None, size=None, dpi=DEFAULT_DPI):
    """
    See :meth:`agatecharts.table.plot`.
    """
    if isinstance(tableset.values()[0], agate.TableSet):
        raise ValueError('agate-charts does not currently support nested TableSets.')

    count = len(tableset)

    if chart.show_legend():
        count += 1

    grid_rows = int(math.sqrt(count))
    grid_columns = math.ceil(float(count) / grid_rows)

    if not size:
        size = (
            DEFAULT_MULTIPLE_SIZE[0] * grid_columns,
            DEFAULT_MULTIPLE_SIZE[1] * grid_rows
        )

    pyplot.figure(figsize=size, dpi=dpi)

    # Compute max domain of all tables so they can be placed on the same axes
    x_min = float('inf')
    x_max = float('-inf')
    y_min = float('inf')
    y_max = float('-inf')

    for table in tableset.values():
        table_x_min, table_x_max = chart.get_x_domain(table)
        table_y_min, table_y_max = chart.get_y_domain(table)

        x_min = min(filter(None, [x_min, table_x_min]))
        x_max = max(filter(None, [x_max, table_x_max]))
        y_min = min(filter(None, [y_min, table_y_min]))
        y_max = max(filter(None, [y_max, table_y_max]))

    x_min, x_max, y_min, y_max = round_limits(x_min, x_max, y_min, y_max)

    i = 0

    for i, (key, table) in enumerate(tableset.items()):
        axes = pyplot.subplot(grid_rows, grid_columns, i + 1)

        legend = chart.plot(table, axes)

        pyplot.title(key)

        pyplot.grid(visible=True, which='major', color='0.85', linestyle='-')
        axes.set_axisbelow(True)

        # matplotlib won't accept Decimal for limit values
        if x_min is not None and x_max is not None:
            axes.set_xlim(float(x_min), float(x_max))

        if y_min is not None and y_max is not None:
            axes.set_ylim(float(y_min), float(y_max))

    if chart.show_legend():
        axes = pyplot.subplot(grid_rows, grid_columns, i + 2)
        pyplot.axis('off')
        axes.legend(*legend, loc='center left', bbox_to_anchor=(0, 0.5))

    pyplot.tight_layout(pad=1, w_pad=1, h_pad=1)

    if filename:
        pyplot.savefig(filename)
    else:
        pyplot.show()


agate.TableSet.bar_chart = bar_chart
agate.TableSet.column_chart = column_chart
agate.TableSet.line_chart = line_chart
agate.TableSet.scatter_chart = scatter_chart



from decimal import ROUND_CEILING, ROUND_FLOOR


def round_limit(n):
    """
    Round a axis minimum or maximum to a suitable break point.
    """
    magnitude = n.log10().to_integral_exact(rounding=ROUND_FLOOR)
    fraction = (n / (10 ** magnitude))

    limit = fraction.to_integral_exact(rounding=ROUND_CEILING) * (10 ** magnitude)

    # If value fits within a half magnitude, break there
    if (fraction % 1 < 0.5):
        limit -= (10 ** magnitude) / 2

    return limit


def round_limits(x_min, x_max, y_min, y_max):
    """
    Round off a set of axis limits so they have appropriate margins.
    """
    if x_min is not None:
        if x_min < 0:
            x_min = round_limit(x_max)
        else:
            x_min = 0

    if x_max is not None and x_max != 0:
        if x_max > 0:
            x_max = round_limit(x_max)
        else:
            x_max = 0

    if y_min is not None:
        if y_min < 0:
            y_min = round_limit(x_max)
        else:
            y_min = 0

    if y_max is not None:
        if y_max > 0:
            y_max = round_limit(y_max)
        else:
            y_max = 0

    return (x_min, x_max, y_min, y_max)



import csv
import os
import unittest

import agate

import agatecharts  # noqa: F401

TEST_FILENAME = '.test.png'


class TestBarsChart(unittest.TestCase):
    def setUp(self):
        text_type = agate.Text()
        number_type = agate.Number()

        column_names = [
            'gender',
            'month',
            'median',
            'stdev',
            '1st',
            '3rd',
            '5th',
            '15th',
            '25th',
            '50th',
            '75th',
            '85th',
            '95th',
            '97th',
            '99th',
        ]
        column_types = [
            text_type,
            number_type,
            number_type,
            number_type,
            number_type,
            number_type,
            number_type,
            number_type,
            number_type,
            number_type,
            number_type,
            number_type,
            number_type,
            number_type,
            number_type,
        ]

        with open('examples/heights.csv') as f:
            # Create a csv reader
            reader = csv.reader(f)

            # Skip header
            next(f)

            # Create the table
            self.table = agate.Table(reader, column_names, column_types)

        if os.path.exists(TEST_FILENAME):
            os.remove(TEST_FILENAME)

    def test_single(self):
        boys = self.table.where(lambda r: r['gender'] == 'male')
        boys.where(lambda r: r['month'] < 73)

        self.table.bar_chart('month', 'median', filename=TEST_FILENAME)

        self.assertTrue(os.path.exists(TEST_FILENAME))

    def test_many(self):
        boys = self.table.where(lambda r: r['gender'] == 'male')
        boys.where(lambda r: r['month'] < 73)

        self.table.bar_chart('month', ['median', 'stdev'], filename=TEST_FILENAME)

        self.assertTrue(os.path.exists(TEST_FILENAME))

    def test_multiples(self):
        genders = self.table.group_by('gender')

        genders.bar_chart('month', 'median', filename=TEST_FILENAME)

        self.assertTrue(os.path.exists(TEST_FILENAME))



import agate

from agatecharts.charts.base import Chart
from agatecharts.colors import Qualitative


class Columns(Chart):
    def __init__(self, label_column_name, value_column_names):
        self._label_column_name = label_column_name

        if isinstance(value_column_names, str):
            value_column_names = [value_column_names]

        self._value_column_names = value_column_names

    def show_legend(self):
        return len(self._value_column_names) > 1

    def get_x_domain(self, table):
        return (None, None)

    def get_y_domain(self, table):
        y_min = min([min(table.columns[name].values_without_nulls()) for name in self._value_column_names])
        y_max = max([max(table.columns[name].values_without_nulls()) for name in self._value_column_names])

        return y_min, y_max

    def plot(self, table, axes):
        label_column = table.columns[self._label_column_name]

        if not isinstance(label_column.data_type, agate.Text) and \
                not isinstance(label_column.data_type, agate.Number) and \
                not isinstance(label_column.data_type, agate.Date):
            raise ValueError('Only Text, Number and Date data are supported for bar chart labels.')

        positions = range(len(label_column))
        colors = Qualitative()
        legend_bars = []
        bar_area = 0.65
        bar_width = bar_area / len(self._value_column_names)

        for i, value_column_name in enumerate(self._value_column_names):
            value_column = table.columns[value_column_name]

            if not isinstance(value_column.data_type, agate.Number):
                raise ValueError('Only Number data is supported for column chart values.')

            series_positions = []

            for j in positions:
                series_positions.append(positions[j] + (i * bar_width))

            plot_bars = axes.bar(
                series_positions,
                value_column,
                bar_width,
                color=next(colors),
                linewidth=0,
                label=value_column_name
            )

            legend_bars.append(plot_bars[0])

        axes.set_xlabel(self._label_column_name)
        axes.set_xticks([p + (bar_area / 2) for p in positions])
        axes.set_xticklabels(table.columns[self._label_column_name])

        if len(self._value_column_names) == 1:
            axes.set_ylabel(self._value_column_names[0])

        return (legend_bars, self._value_column_names)



#!/usr/bin/env python

import os

import agate

import agatecharts

OUTPUT_DIR = 'docs/samples'

if not os.path.exists(OUTPUT_DIR):
    os.mkdir(OUTPUT_DIR)

for filename in os.listdir(OUTPUT_DIR):
    os.remove(os.path.join(OUTPUT_DIR, filename))

tester = agate.TypeTester(force={
    ' Date': agate.Date('%Y-%m-%d')
})

emissions = agate.Table.from_csv('examples/epa-emissions-20150910.csv', tester)

emissions = emissions.compute([
    ('day', agate.Formula(agate.Number(), lambda r: r[' Date'].day)),
    ('so2', agate.Formula(agate.Number(), lambda r: r[' SO2 (tons)'] or 0)),
    ('noX', agate.Formula(agate.Number(), lambda r: r[' NOx (tons)'] or 0)),
    ('co2', agate.Formula(agate.Number(), lambda r: r[' CO2 (short tons)'] or 0))
])

states = emissions.group_by('State')
state_totals = states.aggregate([
    ('so2', agate.Sum('so2')),
    ('co2', agate.Sum('co2')),
    ('noX', agate.Sum('noX'))
])

new_york = states['NY']

# NB: key_type shouldn't be necessary--agate bug #234
days = emissions.group_by('day', key_type=agate.Number())
day_totals = days.aggregate([
    ('so2', agate.Sum('so2')),
    ('co2', agate.Sum('co2')),
    ('noX', agate.Sum('noX'))
])

dates = emissions.group_by(' Date', key_type=agate.Date('%Y-%m-%d'))
date_totals = dates.aggregate([
    ('so2', agate.Sum('so2')),
    ('co2', agate.Sum('co2')),
    ('noX', agate.Sum('noX'))
])

print('Simple charts')

day_totals.line_chart('day', 'co2', filename=os.path.join(OUTPUT_DIR, 'line_chart_simple.png'))
state_totals.column_chart('State', 'co2', filename=os.path.join(OUTPUT_DIR, 'column_chart_simple.png'))
state_totals.bar_chart('State', 'co2', filename=os.path.join(OUTPUT_DIR, 'bar_chart_simple.png'))
day_totals.scatter_chart('so2', 'noX', filename=os.path.join(OUTPUT_DIR, 'scatter_chart.png'))

print('Time series')

date_totals.line_chart(' Date', 'co2', filename=os.path.join(OUTPUT_DIR, 'line_chart_dates.png'))
date_totals.column_chart(' Date', 'co2', filename=os.path.join(OUTPUT_DIR, 'column_chart_dates.png'))
date_totals.bar_chart(' Date', 'co2', filename=os.path.join(OUTPUT_DIR, 'bar_chart_dates.png'))

print('Multiple series')

day_totals.line_chart('day', ['so2', 'noX'], filename=os.path.join(OUTPUT_DIR, 'line_chart_complex.png'))
state_totals.column_chart('State', ['so2', 'noX'], filename=os.path.join(OUTPUT_DIR, 'column_chart_complex.png'))
state_totals.bar_chart('State', ['so2', 'noX'], filename=os.path.join(OUTPUT_DIR, 'bar_chart_complex.png'))

print('Small multiples')

states.line_chart('day', 'co2', filename=os.path.join(OUTPUT_DIR, 'line_chart_simple_multiples.png'))
states.column_chart('day', 'co2', filename=os.path.join(OUTPUT_DIR, 'column_chart_simple_multiples.png'))
states.bar_chart('day', 'co2', filename=os.path.join(OUTPUT_DIR, 'bar_chart_simple_multiples.png'))
states.scatter_chart('so2', 'noX', filename=os.path.join(OUTPUT_DIR, 'scatter_chart_multiples.png'))

print('Small multiples with multiple series')

states.line_chart('day', ['so2', 'noX'], filename=os.path.join(OUTPUT_DIR, 'line_chart_complex_multiples.png'))
states.column_chart('day', ['so2', 'noX'], filename=os.path.join(OUTPUT_DIR, 'column_chart_complex_multiples.png'))
states.bar_chart('day', ['so2', 'noX'], filename=os.path.join(OUTPUT_DIR, 'bar_chart_complex_multiples.png'))



from setuptools import find_packages, setup

with open('README.rst') as f:
    long_description = f.read()

setup(
    name='agate-charts',
    version='0.1.1',
    description='agate-charts adds exploratory charting support to agate.',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    author='Christopher Groskopf',
    author_email='chrisgroskopf@gmail.com',
    url='http://agate-charts.readthedocs.org/',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Multimedia :: Graphics',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Scientific/Engineering :: Visualization',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    packages=find_packages(exclude=['tests', 'tests.*']),
    install_requires=[
        'agate>=0.8.0',
        'matplotlib>=1.4.3'
    ],
    extras_require={
        'test': [
            'pytest',
            'pytest-cov',
        ],
    }
)



import agate

from agatecharts.charts.base import Chart


class Scatter(Chart):
    def __init__(self, x_column_name, y_column_name):
        self._x_column_name = x_column_name
        self._y_column_name = y_column_name

    def show_legend(self):
        return False

    def get_x_domain(self, table):
        x_min = min(table.columns[self._x_column_name])
        x_max = max(table.columns[self._x_column_name])

        return (x_min, x_max)

    def get_y_domain(self, table):
        y_min = min(table.columns[self._y_column_name])
        y_may = max(table.columns[self._y_column_name])

        return (y_min, y_may)

    def plot(self, table, axes):
        x_column = table.columns[self._x_column_name]
        y_column = table.columns[self._y_column_name]

        if not isinstance(x_column.data_type, agate.Number):
            raise ValueError('Only Number data is supported for scatter chart X axis values.')

        if not isinstance(y_column.data_type, agate.Number):
            raise ValueError('Only Number data is supported for scatter chart Y axis values.')

        axes.scatter(
            x_column,
            y_column
        )

        axes.set_xlabel(self._x_column_name)
        axes.set_ylabel(self._y_column_name)
